@@ -1,5 +1,5 @@
 package PAR;
-$PAR::VERSION = '1.004';
+$PAR::VERSION = '1.010';
 
 use 5.006;
 use strict;
@@ -384,7 +384,7 @@ sub import {
         # XXX - handle META.yml here!
         push @PAR_INC, unpar($progname, undef, undef, 1);
 
-        _extract_inc($progname) unless $ENV{PAR_CLEAN};
+        _extract_inc($progname);
         if ($LibCache{$progname}) {
           # XXX bad: this us just a good guess
           require File::Spec;
@@ -494,7 +494,7 @@ sub _import_hash_ref {
         PAR::Heavy::_init_dynaloader();
         
         # XXX - handle META.yml here!
-        _extract_inc($opt->{file}) unless $ENV{PAR_CLEAN};
+        _extract_inc($opt->{file});
         
         my $zip = $LibCache{$opt->{file}};
         my $member = _first_member( $zip,
@@ -537,7 +537,7 @@ sub _import_repository {
     my $obj;
 
     # Support existing clients passed in as objects.
-    if (ref($url) and UNIVERSAL::isa($obj, 'PAR::Repository::Client')) {
+    if (ref($url) and UNIVERSAL::isa($url, 'PAR::Repository::Client')) {
         $obj = $url;
     }
     else {
@@ -550,7 +550,7 @@ sub _import_repository {
     }
 
     if (exists($opt->{fallback}) and not $opt->{fallback}) {
-        push @PriorityRepositoryObjects, $obj; # repository beats local stuff
+        unshift @PriorityRepositoryObjects, $obj; # repository beats local stuff
     } else {
         push @RepositoryObjects, $obj; # local stuff beats repository
     }
@@ -621,7 +621,7 @@ sub _run_member {
 
     if ($is_new) {
         my $file = $member->fileName;
-        print $fh "package main; shift \@INC;\n";
+        print $fh "package main;\n";
         if (defined &Internals::PAR::CLEARSTACK and $clear_stack) {
             print $fh "Internals::PAR::CLEARSTACK();\n";
         }
@@ -630,7 +630,7 @@ sub _run_member {
         seek ($fh, 0, 0);
     }
 
-    unshift @INC, sub { $fh };
+    unshift @INC, sub { shift @INC; return $fh };
 
     $ENV{PAR_0} = $filename; # for Pod::Usage
     { do 'main';
@@ -651,14 +651,14 @@ sub _run_external_file {
     if (defined &Internals::PAR::CLEARSTACK and $clear_stack) {
         $clear_stack = "Internals::PAR::CLEARSTACK();\n";
     }
-    my $string = "package main; shift \@INC;\n$clearstack#line 1 \"$filename\"\n"
+    my $string = "package main;\n$clearstack#line 1 \"$filename\"\n"
                  . do { local $/ = undef; <$ffh> };
     close $ffh;
 
     open my $fh, '<', \$string
       or die "Can't open file handle to string: $!";
 
-    unshift @INC, sub { $fh };
+    unshift @INC, sub { shift @INC; return $fh };
 
     $ENV{PAR_0} = $filename; # for Pod::Usage
     { do 'main';
@@ -673,15 +673,14 @@ sub _run_external_file {
 # returns that directory.
 sub _extract_inc {
     my $file_or_azip_handle = shift;
-    my $force_extract = shift;
-    my $inc = "$PAR::SetupTemp::PARTemp/inc";
     my $dlext = defined($Config{dlext}) ? $Config::Config{dlext} : '';
-    my $inc_exists = -d $inc;
     my $is_handle = ref($file_or_azip_handle) && $file_or_azip_handle->isa('Archive::Zip::Archive');
 
     require File::Spec;
+    my $inc = File::Spec->catdir($PAR::SetupTemp::PARTemp, "inc");
+    my $canary = File::Spec->catfile($PAR::SetupTemp::PARTemp, $PAR::SetupTemp::Canary);
 
-    if (!$inc_exists or $force_extract) {
+    if (!-d $inc || !-e $canary) {
         for (1 .. 10) { mkdir("$inc.lock", 0755) and last; sleep 1 }
         
         undef $@;
@@ -722,17 +721,22 @@ sub _extract_inc {
 
           for ( $zip->memberNames() ) {
               s{^/}{};
-
-              # Skip DLLs (these will be handled by the dynaloader hook) 
-              # except for those placed in File::ShareDir directories.
-              next if (m{\.\Q$dlext\E[^/]*$} && !m{^lib/auto/share/(dist|module)/}); 
-
               my $outfile =  File::Spec->catfile($inc, $_);
               next if -e $outfile and not -w _;
-              $zip->extractMember($_, "$inc/" . $_);
+              $zip->extractMember($_, $outfile);
+              # Unfortunately Archive::Zip doesn't have an option
+              # NOT to restore member timestamps when extracting, hence set 
+              # it to "now" (making it younger than the canary file).
+              utime(undef, undef, $outfile);
           }
         }
         
+        # touch (and back-date) canary file
+        open my $fh, ">", $canary; 
+        close $fh;
+        my $dateback = time() - $PAR::SetupTemp::CanaryDateBack;
+        utime($dateback, $dateback, $canary);
+
         rmdir("$inc.lock");
 
         $ArchivesExtracted{$is_handle ? $file_or_azip_handle->fileName() : $file_or_azip_handle} = $inc;
@@ -793,7 +797,7 @@ sub find_par {
 
     return $rv if defined $rv or not @PriorityRepositoryObjects;
 
-    # the repositories that are prefered over locally installed modules
+    # the repositories that are preferred over locally installed modules
     my $module = $args[1];
     $module =~ s/\.pm$//;
     $module =~ s/\//::/g;
